@@ -1,3 +1,6 @@
+# python imports
+import locale
+
 # django imports
 from django.conf import settings
 from django.core.exceptions import ObjectDoesNotExist
@@ -21,6 +24,7 @@ from lfs.core.signals import cart_changed
 from lfs.core import utils as core_utils
 from lfs.catalog.models import Product
 from lfs.catalog.models import Property
+from lfs.catalog.settings import QUANTITY_FIELD_INTEGER
 from lfs.cart import utils as cart_utils
 from lfs.cart.models import CartItem
 from lfs.core.models import Country
@@ -56,7 +60,7 @@ def cart_inline(request, template_name="lfs/cart/cart_inline.html"):
             "shopping_url": shopping_url,
         }))
 
-    shop = core_utils.get_default_shop()
+    shop = core_utils.get_default_shop(request)
     countries = shop.shipping_countries.all()
     selected_country = shipping_utils.get_selected_shipping_country(request)
 
@@ -76,7 +80,7 @@ def cart_inline(request, template_name="lfs/cart/cart_inline.html"):
     # Discounts
     discounts = lfs.discounts.utils.get_valid_discounts(request)
     for discount in discounts:
-        cart_price = cart_price - discount["price"]
+        cart_price = cart_price - discount["price_gross"]
 
     # Voucher
     voucher_number = lfs.voucher.utils.get_current_voucher_number(request)
@@ -89,27 +93,28 @@ def cart_inline(request, template_name="lfs/cart/cart_inline.html"):
         voucher_message = MESSAGES[6]
     else:
         lfs.voucher.utils.set_current_voucher_number(request, voucher_number)
-        is_voucher_effective, voucher_message = voucher.is_effective(request, cart)
+        is_voucher_effective, voucher_message = voucher.is_effective(cart)
         if is_voucher_effective:
             display_voucher = True
-            voucher_value = voucher.get_price_gross(request, cart)
+            voucher_value = voucher.get_price_gross(cart)
             cart_price = cart_price - voucher_value
-            voucher_tax = voucher.get_tax(request, cart)
+            voucher_tax = voucher.get_tax(cart)
         else:
             display_voucher = False
             voucher_value = 0
             voucher_tax = 0
 
+    # Calc delivery time for cart (which is the maximum of all cart items)
     max_delivery_time = cart.get_delivery_time(request)
 
-    # Calc delivery date for cart (which is the maximum of all cart items)
-    max_delivery_date = cart.get_delivery_time(request)
-
     cart_items = []
     for cart_item in cart.get_items():
+        product = cart_item.product
+        quantity = product.get_clean_quantity(cart_item.amount)
         cart_items.append({
             "obj": cart_item,
-            "product": cart_item.product,
+            "quantity": quantity,
+            "product": product,
             "product_price_net": cart_item.get_price_net(request),
             "product_price_gross": cart_item.get_price_gross(request),
             "product_tax": cart_item.get_tax(request),
@@ -117,7 +122,6 @@ def cart_inline(request, template_name="lfs/cart/cart_inline.html"):
 
     return render_to_string(template_name, RequestContext(request, {
         "cart_items": cart_items,
-        "max_delivery_date": max_delivery_date,
         "cart_price": cart_price,
         "cart_tax": cart_tax,
         "shipping_methods": shipping_utils.get_valid_shipping_methods(request),
@@ -167,18 +171,21 @@ def added_to_cart_items(request, template_name="lfs/cart/added_to_cart_items.htm
     cart_items = []
     for cart_item in request.session.get("cart_items", []):
         total += cart_item.get_price_gross(request)
+        product = cart_item.product
+        quantity = product.get_clean_quantity(cart_item.amount)
         cart_items.append({
-            "product": cart_item.product,
+            "product": product,
             "obj": cart_item,
+            "quantity": quantity,
             "product_price_net": cart_item.get_price_net(request),
             "product_price_gross": cart_item.get_price_gross(request),
             "product_tax": cart_item.get_tax(request),
         })
 
-    return render_to_string(template_name, {
+    return render_to_string(template_name, RequestContext(request, {
         "total": total,
         "cart_items": cart_items,
-    })
+    }))
 
 
 # Actions
@@ -228,9 +235,9 @@ def add_to_cart(request, product_id=None):
         raise Http404()
 
     try:
-        quantity = float(request.POST.get("quantity", 1))
+        quantity = locale.atof(request.POST.get("quantity", "1.0"))
     except (TypeError, ValueError):
-        quantity = 1
+        quantity = 1.0
 
     # Validate properties (They are added below)
     properties_dict = {}
@@ -247,7 +254,10 @@ def add_to_cart(request, product_id=None):
                     continue
 
                 if property.is_number_field:
-                    value = l10n_float(value)
+                    try:
+                        value = locale.atof(value)
+                    except ValueError:
+                        value = locale.atof("0.0")
 
                 properties_dict[property_id] = unicode(value)
 
@@ -273,10 +283,6 @@ def add_to_cart(request, product_id=None):
                         return lfs.core.utils.set_message_cookie(
                             product.get_absolute_url(), msg)
 
-    elif product.is_product_with_variants():
-        variant_id = request.POST.get("variant_id")
-        product = lfs_get_object_or_404(Product, pk=variant_id)
-
     if product.active_packing_unit:
         quantity = lfs.catalog.utils.calculate_real_amount(product, quantity)
 
@@ -313,7 +319,7 @@ def add_to_cart(request, product_id=None):
             except TypeError:
                 quantity = 1
 
-            cart_item = cart.add(product=accessory, amount=1)
+            cart_item = cart.add(product=accessory, amount=quantity)
             cart_items.append(cart_item)
 
     # Store cart items for retrieval within added_to_cart.
@@ -382,23 +388,23 @@ def refresh_cart(request):
     # Update Amounts
     message = ""
     for item in cart.get_items():
-        amount = request.POST.get("amount-cart-item_%s" % item.id, 0)
         try:
-            amount = float(amount)
-            if item.product.manage_stock_amount and amount > item.product.stock_amount and not item.product.order_time:
-                amount = item.product.stock_amount
-                if amount < 0:
-                    amount = 0
+            amount = locale.atof(request.POST.get("amount-cart-item_%s" % item.id, "0.0"))
+        except (TypeError, ValueError):
+            amount = 1.0
 
-                if amount == 0:
-                    message = _(u"Sorry, but '%(product)s' is not available anymore." % {"product": item.product.name})
-                elif amount == 1:
-                    message = _(u"Sorry, but '%(product)s' is only one time available." % {"product": item.product.name})
-                else:
-                    message = _(u"Sorry, but '%(product)s' is only %(amount)s times available.") % {"product": item.product.name, "amount": amount}
+        if item.product.manage_stock_amount and amount > item.product.stock_amount and not item.product.order_time:
+            amount = item.product.stock_amount
+            if amount < 0:
+                amount = 0
+
+            if amount == 0:
+                message = _(u"Sorry, but '%(product)s' is not available anymore." % {"product": item.product.name})
+            elif amount == 1:
+                message = _(u"Sorry, but '%(product)s' is only one time available." % {"product": item.product.name})
+            else:
+                message = _(u"Sorry, but '%(product)s' is only %(amount)s times available.") % {"product": item.product.name, "amount": amount}
 
-        except ValueError:
-            amount = 1
 
         if item.product.active_packing_unit:
             item.amount = lfs.catalog.utils.calculate_real_amount(item.product, float(amount))
