@@ -5,6 +5,7 @@ import urllib
 # django imports
 from django.conf import settings
 from django.core.cache import cache
+from django.core.paginator import Paginator, EmptyPage, InvalidPage
 from django.core.urlresolvers import reverse
 from django.http import Http404
 from django.http import HttpResponse
@@ -25,12 +26,14 @@ from lfs.catalog.models import File
 from lfs.catalog.models import Product
 from lfs.catalog.models import ProductPropertyValue
 from lfs.catalog.models import PropertyOption
+from lfs.catalog.models import ProductAttachment
+from lfs.catalog.settings import CATEGORY_VARIANT_CHEAPEST_PRICES
+from lfs.catalog.settings import CONTENT_PRODUCTS
 from lfs.catalog.settings import PRODUCT_WITH_VARIANTS
-from lfs.catalog.settings import VARIANT
 from lfs.catalog.settings import PROPERTY_VALUE_TYPE_DEFAULT
 from lfs.catalog.settings import SELECT
-from lfs.catalog.settings import CONTENT_PRODUCTS
-from lfs.core.utils import LazyEncoder
+from lfs.catalog.settings import VARIANT
+from lfs.core.utils import LazyEncoder, lfs_pagination
 from lfs.core.templatetags import lfs_tags
 from lfs.utils import misc as lfs_utils
 
@@ -50,10 +53,11 @@ def select_variant(request):
     changed.
     """
     variant_id = request.POST.get("variant_id")
+    variant = Product.objects.get(pk=variant_id)
     msg = _(u"The product has been changed according to your selection.")
 
     result = simplejson.dumps({
-        "product": product_inline(request, variant_id),
+        "product": product_inline(request, variant),
         "message": msg,
     }, cls=LazyEncoder)
 
@@ -67,16 +71,18 @@ def calculate_packing(request, id, quantity=None, as_string=False, template_name
     product = Product.objects.get(pk=id)
 
     if quantity is None:
-        try:
-            quantity = float(request.POST.get("quantity"))
-        except (TypeError, ValueError):
-            quantity = 1
+        quantity = float(request.POST.get("quantity"))
 
     packing_amount, packing_unit = product.get_packing_info()
 
-    packs = math.ceil(quantity / packing_amount)
-    real_quantity = packs * packing_amount
-    price = real_quantity * product.get_price(request)
+    try:
+        packs = math.ceil(quantity / packing_amount)
+        real_quantity = packs * packing_amount
+        price = real_quantity * product.get_price_gross(request)
+    except TypeError:
+        packs = 0.0
+        real_quantity = 0.0
+        price = 0.0
 
     html = render_to_string(template_name, RequestContext(request, {
         "price": price,
@@ -130,7 +136,7 @@ def calculate_price(request, id):
     price += property_price
 
     result = simplejson.dumps({
-        "price": lfs_tags.currency(price),
+        "price": lfs_tags.currency(price, request),
         "for-sale-standard-price": lfs_tags.currency(for_sale_standard_price),
         "for-sale-price": lfs_tags.currency(for_sale_price),
         "message": _("Price has been changed according to your selection."),
@@ -140,15 +146,18 @@ def calculate_price(request, id):
 
 
 def select_variant_from_properties(request):
-    """This is called via an ajax call if the combination of properties are
+    """
+    This is called via an ajax call if the combination of properties are
     changed.
     """
     product_id = request.POST.get("product_id")
 
     try:
-        product = Product.objects.get(pk=product_id)
-    except:  # Product.DoesNotExist:
+        variant = Product.objects.get(pk=product_id)
+    except Product.DoesNotExist:
         return HttpResponse("")
+    else:
+        product = variant.parent
 
     options = lfs_utils.parse_properties(request)
     variant = product.get_variant(options)
@@ -160,7 +169,7 @@ def select_variant_from_properties(request):
         msg = _(u"The product has been changed according to your selection.")
 
     result = simplejson.dumps({
-        "product": product_inline(request, variant.id),
+        "product": product_inline(request, variant),
         "message": msg,
     }, cls=LazyEncoder)
 
@@ -193,12 +202,12 @@ def set_price_filter(request, category_slug):
 
     try:
         float(min)
-    except TypeError:
+    except (TypeError, ValueError):
         min = "0"
 
     try:
         float(max)
-    except TypeError:
+    except (TypeError, ValueError):
         max = "0"
 
     request.session["price-filter"] = {"min": min, "max": max}
@@ -259,7 +268,7 @@ def set_sorting(request):
 def category_view(request, slug, template_name="lfs/catalog/category_base.html"):
     """
     """
-    start = request.REQUEST.get("start", 0)
+    start = request.REQUEST.get("start", 1)
     category = lfs_get_object_or_404(Category, slug=slug)
     if category.get_content() == CONTENT_PRODUCTS:
         inline = category_products(request, slug, start)
@@ -319,7 +328,7 @@ def category_categories(request, slug, start=0, template_name="lfs/catalog/categ
     return result
 
 
-def category_products(request, slug, start=0, template_name="lfs/catalog/categories/product/default.html"):
+def category_products(request, slug, start=1, template_name="lfs/catalog/categories/product/default.html"):
     """Displays the products of the category with passed slug.
 
     This view is called if the user chooses a template that is situated in settings.PRODUCT_PATH ".
@@ -368,7 +377,7 @@ def category_products(request, slug, start=0, template_name="lfs/catalog/categor
     try:
         start = int(start)
     except (ValueError, TypeError):
-        start = 0
+        start = 1
 
     format_info = category.get_format_info()
     amount_of_rows = format_info["product_rows"]
@@ -378,13 +387,23 @@ def category_products(request, slug, start=0, template_name="lfs/catalog/categor
     all_products = lfs.catalog.utils.get_filtered_products_for_category(
         category, product_filter, price_filter, sorting)
 
+    # prepare paginator
+    paginator = Paginator(all_products, amount)
+
+    try:
+        current_page = paginator.page(start)
+    except (EmptyPage, InvalidPage):
+        current_page = paginator.page(paginator.num_pages)
+
     # Calculate products
     row = []
     products = []
-    for i, product in enumerate(all_products[start:start + amount]):
-        # Switch to default variant if product is a "product with variants".
-        # if product.is_product_with_variants() and product.has_variants():
-        #    product = product.get_default_variant()
+    for i, product in enumerate(current_page.object_list):
+        if product.is_product_with_variants():
+            default_variant = product.get_variant_for_category(request)
+            if default_variant:
+                product = default_variant
+
         image = None
         product_image = product.get_image()
         if product_image:
@@ -394,12 +413,8 @@ def category_products(request, slug, start=0, template_name="lfs/catalog/categor
             "slug": product.slug,
             "name": product.get_name(),
             "image": image,
-            "price": product.get_price(request),
-            "standard_price": product.get_standard_price(request),
-            "for_sale_price": product.for_sale_price,
-            "get_for_sale": product.get_for_sale(),
             "price_unit": product.price_unit,
-            "price_includes_tax": product.price_includes_tax,
+            "price_includes_tax": product.price_includes_tax(request),
         })
         if (i + 1) % amount_of_cols == 0:
             products.append(row)
@@ -411,24 +426,7 @@ def category_products(request, slug, start=0, template_name="lfs/catalog/categor
     amount_of_products = all_products.count()
 
     # Calculate urls
-    pages = []
-    for i in range(0, amount_of_products / amount + 1):
-        page_start = i * amount
-        pages.append({
-            "name": i + 1,
-            "start": page_start,
-            "selected": start == page_start,
-        })
-
-    if (start + amount) < amount_of_products:
-        next_url = "%s?start=%s" % (category.get_absolute_url(), start + amount)
-    else:
-        next_url = None
-
-    if (start - amount) >= 0:
-        previous_url = "%s?start=%s" % (category.get_absolute_url(), start - amount)
-    else:
-        previous_url = None
+    pagination_data = lfs_pagination(request, current_page, url=category.get_absolute_url())
 
     render_template = category.get_template_name()
     if render_template != None:
@@ -437,11 +435,8 @@ def category_products(request, slug, start=0, template_name="lfs/catalog/categor
     result = render_to_string(template_name, RequestContext(request, {
         "category": category,
         "products": products,
-        "next_url": next_url,
-        "previous_url": previous_url,
         "amount_of_products": amount_of_products,
-        "pages": pages,
-        "show_pages": amount_of_products > amount,
+        "pagination": pagination_data,
         "all_products": all_products,
     }))
 
@@ -468,48 +463,44 @@ def product_view(request, slug, template_name="lfs/catalog/product_base.html"):
     request.session["RECENT_PRODUCTS"] = recent
 
     result = render_to_response(template_name, RequestContext(request, {
-        "product_inline": product_inline(request, product.id),
+        "product_inline": product_inline(request, product),
         "product": product,
     }))
 
     return result
 
 
-def product_inline(request, id, template_name="lfs/catalog/products/product_inline.html"):
-    """Part of the prduct view, which displays the actual data of the product.
+def product_inline(request, product, template_name="lfs/catalog/products/product_inline.html"):
+    """
+    Part of the product view, which displays the actual data of the product.
 
     This is factored out to be able to better cached and in might in future used
     used to be updated via ajax requests.
     """
-    cache_key = "%s-product-inline-%s-%s" % (settings.CACHE_MIDDLEWARE_KEY_PREFIX, request.user.is_superuser, id)
+    cache_key = "%s-product-inline-%s-%s" % (settings.CACHE_MIDDLEWARE_KEY_PREFIX, request.user.is_superuser, product.id)
     result = cache.get(cache_key)
     if result is not None:
         return result
 
-    # Get product in question
-    product = lfs_get_object_or_404(Product, pk=id)
-
-    if product.sub_type == PRODUCT_WITH_VARIANTS:
-        variant = product.get_default_variant()
-        if variant is None:
-            variant = product
-    elif product.sub_type == VARIANT:
-        variant = product
-        product = product.parent
-    else:
-        variant = product
+    # Switching to default variant
+    if product.is_product_with_variants():
+        temp = product.get_default_variant()
+        product = temp if temp else product
 
     properties = []
     variants = []
 
-    if product.is_product_with_variants():
-        if product.variants_display_type == SELECT:
+    display_variants_list = True
+    if product.is_variant():
+        parent = product.parent
+        if parent.variants_display_type == SELECT:
+            display_variants_list = False
             # Get all properties (sorted). We need to traverse through all
             # property/options to select the options of the current variant.
-            for property in product.get_property_select_fields():
+            for property in parent.get_property_select_fields():
                 options = []
                 for property_option in property.options.all():
-                    if variant.has_option(property, property_option):
+                    if product.has_option(property, property_option):
                         selected = True
                     else:
                         selected = False
@@ -526,13 +517,12 @@ def product_inline(request, id, template_name="lfs/catalog/products/product_inli
                     "options": options,
                 })
         else:
-            properties = product.get_property_select_fields()
-            variants = product.get_variants()
+            properties = parent.get_property_select_fields()
+            variants = parent.get_variants()
 
     elif product.is_configurable_product():
         for property in product.get_configurable_properties():
             options = []
-
             try:
                 ppv = ProductPropertyValue.objects.get(product=product, property=property, type=PROPERTY_VALUE_TYPE_DEFAULT)
                 ppv_value = ppv.value
@@ -552,7 +542,6 @@ def product_inline(request, id, template_name="lfs/catalog/products/product_inli
                     "price": property_option.price,
                     "selected": selected,
                 })
-
             properties.append({
                 "obj": property,
                 "id": property.id,
@@ -567,20 +556,27 @@ def product_inline(request, id, template_name="lfs/catalog/products/product_inli
     if product.get_template_name() != None:
         template_name = product.get_template_name()
 
-    if variant.active_packing_unit:
-        packing_result = calculate_packing(request, variant.id, 1, True)
+    if product.get_active_packing_unit():
+        packing_result = calculate_packing(request, product.id, 1, True)
     else:
         packing_result = ""
 
+    # attachments
+    attachments = product.get_attachments()
+
     result = render_to_string(template_name, RequestContext(request, {
         "product": product,
-        "standard_price": variant.get_standard_price(request),
-        "price": variant.get_price(request),
-        "variant": variant,
         "variants": variants,
-        "product_accessories": variant.get_accessories(),
+        "product_accessories": product.get_accessories(),
         "properties": properties,
         "packing_result": packing_result,
+        "attachments": attachments,
+        "quantity": product.get_clean_quantity(1),
+        "price_includes_tax": product.price_includes_tax(request),
+        "price_unit": product.get_price_unit(),
+        "unit": product.get_unit(),
+        "display_variants_list": display_variants_list,
+        "for_sale": product.for_sale,
     }))
 
     cache.set(cache_key, result)
