@@ -2,6 +2,8 @@
 import datetime
 import sys
 import urllib
+from collections import deque
+from itertools import count
 
 # django imports
 from django.conf import settings
@@ -33,13 +35,23 @@ def l10n_float(string):
         return 0.0
 
 
-def get_default_shop():
+def get_default_shop(request=None):
     """Returns the default shop.
     """
+    if request:
+        try:
+            return request.shop
+        except AttributeError:
+            pass
+
     try:
         shop = Shop.objects.get(pk=1)
     except Shop.DoesNotExist, e:  # No guarantee that our shop will have pk=1 in postgres
         shop = Shop.objects.all()[0]
+
+    if request:
+        request.shop = shop
+
     return shop
 
 
@@ -61,6 +73,14 @@ def import_module(module):
     return module
 
 
+def import_symbol(symbol):
+    """Imports symbol with given dotted name.
+    """
+    module_str, symbol_str = symbol.rsplit('.', 1)
+    module = import_module(module_str)
+    return getattr(module, symbol_str)
+
+
 def set_message_to(response, msg):
     """Sets message cookie with passed message to passed response.
     """
@@ -286,3 +306,95 @@ class CategoryTree(object):
             })
 
         return categories
+
+
+def define_page_range(current_page, total_pages, window=6):
+    """ Returns range of pages that contains current page and few pages before and after it.
+
+        @current_page - starts from 1
+        @tota_pages - total number of pages
+        @window - maximum number of pages shown with current page - should be even
+
+        Examples (cucumber style):
+             Given window = 6
+             When current_page is 8
+             and total_pages = 20
+             Then I should see: 5 6 7 [8] 9 10 11
+
+             Given window = 6
+             When current_page is 8
+             and total_pages = 9
+             Then I should see: 3 4 5 6 7 [8] 9
+
+             Given window = 6
+             When current_page is 1
+             and total_pages = 9
+             Then I should see: [1] 2 3 4 5 6 7
+    """
+    # maximum length of page range is window + 1
+    maxlen = window + 1
+    page_range = deque(maxlen=maxlen)
+
+    # minimum possible index is either: (current_page - window) or 1
+    window_start = (current_page - window) if (current_page - window) > 0 else 1
+
+    # maximum possible index is current_page + window or total_pages
+    window_end = total_pages if (current_page + window) > total_pages else (current_page + window)
+
+    # if we have enough pages then we should end at preffered end
+    preffered_end = current_page + int(window / 2.0)
+
+    for i in count(window_start):
+        if i > window_end:
+            # if we're on first page then our window will be [1] 2 3 4 5 6 7
+            break
+        elif i > preffered_end and len(page_range) == maxlen:
+            # if we have enough pages already then stop at preffered_end
+            break
+        page_range.append(i)
+    return list(page_range)
+
+
+def lfs_pagination(request, current_page, url='', getparam='start'):
+    """Prepare data for pagination
+
+       @page - number of current page (starting from 1)
+       @paginator - paginator object, eg. Paginator(contact_list, 25)
+    """
+    paginator = current_page.paginator
+    current_page_no = current_page.number
+
+    has_next = current_page.has_next()
+    has_prev = current_page.has_previous()
+
+    page_range = define_page_range(current_page.number, paginator.num_pages)
+
+    first = 1
+    last = paginator.num_pages
+
+    if first in page_range:
+        first = None
+
+    if last in page_range:
+        last = None
+
+    to_return = {'page_range': page_range,
+                 'current_page': current_page_no,
+                 'total_pages': paginator.num_pages,
+                 'has_next': has_next,
+                 'has_prev': has_prev,
+                 'next': current_page_no + 1,
+                 'prev': current_page_no - 1,
+                 'url': url,
+                 'getparam': getparam,
+                 'first_page': first,
+                 'last_page': last,
+                 'getvars': ''
+                }
+
+    getvars = request.GET.copy()
+    if getparam in getvars:
+        del getvars[getparam]
+    if len(getvars.keys()) > 0:
+        to_return['getvars'] = "&%s" % getvars.urlencode()
+    return to_return
\ No newline at end of file
