@@ -1,8 +1,14 @@
+# django imports
+from django.conf import settings
+
 # lfs imports
 import lfs.discounts.utils
 import lfs.voucher.utils
 from lfs.cart import utils as cart_utils
 from lfs.core.models import Country
+from lfs.core.signals import order_created
+from lfs.core.utils import import_module
+from lfs.core.utils import import_symbol
 from lfs.customer import utils as customer_utils
 from lfs.order.models import Order
 from lfs.order.models import OrderItem
@@ -11,7 +17,6 @@ from lfs.payment import utils as payment_utils
 from lfs.shipping import utils as shipping_utils
 from lfs.voucher.models import Voucher
 
-
 def add_order(request):
     """Adds an order based on current cart for the current customer.
 
@@ -22,10 +27,10 @@ def add_order(request):
     order = None
 
     invoice_address = customer.selected_invoice_address
-    if customer.selected_shipping_address:
-        shipping_address = customer.selected_shipping_address
-    else:
+    if request.POST.get("no_shipping"):
         shipping_address = customer.selected_invoice_address
+    else:
+        shipping_address = customer.selected_shipping_address
 
     cart = cart_utils.get_cart(request)
     if cart is None:
@@ -68,11 +73,11 @@ def add_order(request):
     except Voucher.DoesNotExist:
         voucher = None
     else:
-        is_voucher_effective, voucher_message = voucher.is_effective(request, cart)
+        is_voucher_effective, voucher_message = voucher.is_effective(cart)
         if is_voucher_effective:
             voucher_number = voucher.number
-            voucher_price = voucher.get_price_gross(request, cart)
-            voucher_tax = voucher.get_tax(request, cart)
+            voucher_price = voucher.get_price_gross(cart)
+            voucher_tax = voucher.get_tax(cart)
 
             price -= voucher_price
             tax -= voucher_tax
@@ -98,8 +103,9 @@ def add_order(request):
 
         invoice_firstname=customer.selected_invoice_address.firstname,
         invoice_lastname=customer.selected_invoice_address.lastname,
-        invoice_line1=invoice_address.company_name,
-        invoice_line2=invoice_address.street,
+        invoice_company_name=customer.selected_invoice_address.company_name,
+        invoice_line1=invoice_address.line1,
+        invoice_line2=invoice_address.line2,
         invoice_city=invoice_address.city,
         invoice_state=invoice_address.state,
         invoice_code=invoice_address.zip_code,
@@ -108,8 +114,9 @@ def add_order(request):
 
         shipping_firstname=shipping_address.firstname,
         shipping_lastname=shipping_address.lastname,
-        shipping_line1=shipping_address.company_name,
-        shipping_line2=shipping_address.street,
+        shipping_company_name=shipping_address.company_name,
+        shipping_line1=shipping_address.line1,
+        shipping_line2=shipping_address.line2,
         shipping_city=shipping_address.city,
         shipping_state=shipping_address.state,
         shipping_code=shipping_address.zip_code,
@@ -182,10 +189,27 @@ def add_order(request):
             product_tax=-discount["tax"],
         )
 
+    # Send signal before cart is deleted.
+    order_created.send({"order":order, "cart": cart, "request": request})
+
     cart.delete()
 
     # Note: Save order for later use in thank you page. The order will be
     # removed from the session if the thank you page has been called.
     request.session["order"] = order
 
+    ong = import_symbol(settings.LFS_ORDER_NUMBER_GENERATOR)
+    try:
+        order_numbers = ong.objects.get(id="order_number")
+    except ong.DoesNotExist:
+        order_numbers = ong.objects.create(id="order_number")
+
+    try:
+        order_numbers.init(request, order)
+    except AttributeError:
+        pass
+
+    order.number = order_numbers.get_next()
+    order.save()
+
     return order
