@@ -50,6 +50,35 @@ function send_form_and_refresh(mythis) {
     })
 }
 
+function sortable() {
+    $('ul.sortable').sortable({
+        placeholder: 'placeholder',
+        forcePlaceholderSize: true,
+        handle: '.handle',
+        helper: 'clone',
+        items: 'li',
+        opacity: .6,
+        revert: 250,
+        tabSize: 25,
+        tolerance: 'pointer',
+        toleranceElement: '> div',
+        stop: function(event, ui){
+            var url = $(this).attr("href");
+            var serialized = ui.item.closest('ul.sortable').sortable('serialize')
+            $.ajax({
+                url: url,
+                context: document.body,
+                type: "POST",
+                data: {"objs": serialized},
+                success: function(data) {
+                    data = $.parseJSON(data);
+                    $.jGrowl(data["message"])
+                }
+           });
+        }
+    });
+}
+
 $(function() {
     update_editor();
     $(".button").button();
@@ -73,9 +102,12 @@ $(function() {
 
     // General product tabs
     $('#manage-tabs').tabs();
+    $('#manage-tabs').show();
 
     // Product tabs
     $('#product-tabs').tabs();
+    $('#product-tabs').show();
+
     $('#product-tabs').bind('tabsshow', function(event, ui) {
         $.cookie("product_tab", ui.index);
     });
@@ -85,6 +117,7 @@ $(function() {
 
     // Category tabs
     $('#category-tabs').tabs();
+    $('#category-tabs').show();
     $('#category-tabs').bind('tabsshow', function(event, ui) {
         $.cookie("category_tab", ui.index);
     });
@@ -135,9 +168,6 @@ $(function() {
                 if (data["message"]) {
                     $.jGrowl(data["message"]);
                 }
-                if (data["init_date"]) {
-                    DateTimeShortcuts.init();
-                }
                 hide_ajax_loading();
                 update_editor();
 
@@ -185,6 +215,13 @@ $(function() {
         });
     });
 
+    $(".toggle-all").live("click", function() {
+        var selector = ".toggle-" + $(this).attr("data")
+        $(selector).each(function() {
+            this.checked = !this.checked;
+        });
+    });
+
     // Criteria
     $(".criterion-add-first-button").live("click", function() {
         var position = $(this).siblings(".position").val()
@@ -225,6 +262,7 @@ $(function() {
         $.get(url, function(data) {
             $("#dialog").html(data);
             $("#dialog").dialog("open");
+            addEditor('#id_portlet-text', true, 300);
         });
         return false;
     });
@@ -234,6 +272,7 @@ $(function() {
             success : function(data) {
                 $("#dialog").html(data);
                 $("#dialog").dialog("open");
+                addEditor('#id_portlet-text', true, 300);
         }});
         return false;
     });
@@ -391,6 +430,59 @@ $(function() {
         }
         return false;
     })
+
+    $("input.date-picker").datepicker({
+        dateFormat: 'yy-mm-dd',
+        showWeek: true,
+        firstDay: 1
+    });
+
+    $('ol.sortable').nestedSortable({
+        placeholder: 'placeholder',
+        forcePlaceholderSize: true,
+        handle: '.handle',
+        helper: 'clone',
+        items: 'li',
+        opacity: .6,
+        revert: 250,
+        tabSize: 25,
+        tolerance: 'pointer',
+        toleranceElement: '> div',
+        stop: function(event, ui){
+            var url = $(this).attr("href");
+            serialized = $('ol.sortable').nestedSortable('serialize');
+            $.ajax({
+                url: url,
+                context: document.body,
+                type: "POST",
+                data: {"categories": serialized},
+                success: function(data) {
+                    data = $.parseJSON(data);
+                    $.jGrowl(data["message"])
+                }
+           });
+        }
+    });
+
+    $("#insert-image").live("click", function(e) {
+        var url = $("input.image:checked").attr("value");
+        var size = $("#image-size").val();
+        var klass = $("#image-class").val();
+
+        if (size)
+            url = url.replace("100x100", size);
+        else
+            url = url.replace(".100x100", "");
+
+        if (klass)
+            html = "<img class='" + klass + "' src='" + url + "' />"
+        else
+            html = "<img src='" + url + "' />"
+
+        insertHTML(html);
+        $("#dialog").dialog("close");
+        return false;
+    })
 })
 
 $(document).ajaxComplete(function() {
